/*
 *  Copyright 2015  Andreas Cord-Landwehr <cordlandwehr@kde.org>
 *
 *  SPDX-License-Identifier: LGPL-2.1-or-later
 */

#ifndef ARTIKULATE_DEBUG_H
#define ARTIKULATE_DEBUG_H

#include <QLoggingCategory>
Q_DECLARE_LOGGING_CATEGORY(ARTIKULATE_LOG)
Q_DECLARE_LOGGING_CATEGORY(ARTIKULATE_CORE)
Q_DECLARE_LOGGING_CATEGORY(ARTIKULATE_PARSER)

#endif
