/*
 *  Copyright 2015  Andreas Cord-Landwehr <cordlandwehr@kde.org>
 *
 *  SPDX-License-Identifier: LGPL-2.1-or-later
 */

#include "libsound_debug.h"
Q_LOGGING_CATEGORY(LIBSOUND_LOG, "log_libsound")
